import java.util.Map;

Table overall_data;
int rows, cols;
int circle_center_x, circle_center_y, diameter;

String[] itemType;
int[] itemNumber;
int[] itemCheckouts;

float mean;
int total, max, median;

HashMap<String, Integer> colors = new HashMap<String, Integer>();
HashMap<String, Integer> counts = new HashMap<String, Integer>();
HashMap<String, Integer> checkouts = new HashMap<String, Integer>();

void setup() {
  size(displayWidth, displayHeight);
  overall_data = loadTable("xCatCF_SeattlePublicLibrary.csv", "header");
  rows = overall_data.getRowCount(); 
  cols = overall_data.getColumnCount();

  itemType = new String[rows];
  itemNumber = new int[rows];
  itemCheckouts = new int[rows];
  total = 0;
  max = 0;
  
  colors.put("jcvhs", color(75, 180, 80));
  colors.put("jcdvd", color(230, 85, 70));
  colors.put("jccd", color(230, 225, 113));
  colors.put("jcbk", color(70, 100, 175));
  
  counts.put("jcvhs", 0);
  counts.put("jcdvd", 0);
  counts.put("jccd", 0);
  counts.put("jcbk", 0);
  
  checkouts.put("jcvhs", 0);
  checkouts.put("jcdvd", 0);
  checkouts.put("jccd", 0);
  checkouts.put("jcbk", 0);

  for (int i=0; i<rows; i++) {
    itemType[i] = overall_data.getString(i, 0);
    itemNumber[i] = overall_data.getInt(i, 1);
    itemCheckouts[i] = overall_data.getInt(i, 2);
    total += itemCheckouts[i];

    if (itemCheckouts[i] > max) {
      max = itemCheckouts[i];
    }
    
    counts.put(itemType[i], counts.get(itemType[i]) + 1);
    checkouts.put(itemType[i], checkouts.get(itemType[i]) + itemCheckouts[i]);
  }

  mean = total / rows;
  
  // Find the median checkout count.
  int[] checkoutSorted = new int[rows];
  arrayCopy(itemCheckouts, checkoutSorted);
  sort(checkoutSorted);
  
  median = checkoutSorted[int(rows / 2)];
}

void draw() {  
  background(64);
  strokeCap(ROUND);

  circle_center_x = (displayWidth/2)+250;
  circle_center_y = displayHeight/2;
  diameter = displayWidth/3;
  
  fill(255);
  textSize(32);
  text("Charlie and the Chocolate Factory: Number of Checkouts by Item Type", displayWidth / 2, 10);
  
  int top, left;
  float ratio;
  
  textAlign(CENTER, TOP);
  rectMode(CORNER);
  
  // Data
  top = left = 150;
  
  textSize(24);
  fill(200);
  text("Data", left + 125, top - 60);

  textSize(14);
  
  fill(colors.get("jcbk"));
  ratio = (float)counts.get("jcbk") / rows;
  text("books", left, top);
  text(nf(ratio * 100, 0, 1) + "%", left+50, top);
  text(counts.get("jcbk") + " copies", left+125, top);
  rect(left+170, top, ratio * 250, 15);
  
  fill(colors.get("jccd"));
  ratio = (float)counts.get("jccd") / rows;
  text("cds", left, top+30);
  text(nf(ratio * 100, 0, 1) + "%", left+50, top+30);
  text(counts.get("jccd") + " copies", left+125, top+30);
  rect(left+170, top+30, ratio * 250, 15);
  
  fill(colors.get("jcdvd"));
  ratio = (float)counts.get("jcdvd") / rows;
  text("dvds", left, top+60);
  text(nf(ratio * 100, 0, 1) + "%", left+50, top+60);
  text(counts.get("jcdvd") + " copies", left+125, top+60);
  rect(left+170, top+60, ratio * 250, 15);
  
  fill(colors.get("jcvhs"));
  ratio = (float)counts.get("jcvhs") / rows;
  text("vhs", left, top+90);
  text(nf(ratio * 100, 0, 1) + "%", left+50, top+90);
  text(counts.get("jcvhs") + " copies", left+125, top+90);
  rect(left+170, top+90, ratio * 250, 15);
  
  // Analysis
  top = circle_center_y;
  
  textSize(24);
  fill(200);
  text("Analysis", left + 125, top - 60);

  textSize(14);
  
  fill(colors.get("jcbk"));
  ratio = (float)checkouts.get("jcbk") / total;
  text("books", left, top);
  text(nf(ratio * 100, 0, 1) + "%", left+50, top);
  text(checkouts.get("jcbk") + "  c/o", left+125, top);
  rect(left+170, top, ratio * 250, 15);
  
  fill(colors.get("jccd"));
  ratio = (float)checkouts.get("jccd") / total;
  text("cds", left, top+30);
  text(nf(ratio * 100, 0, 1) + "%", left+50, top+30);
  text(checkouts.get("jccd") + " c/o", left+125, top+30);
  rect(left+170, top+30, ratio * 250, 15);
  
  fill(colors.get("jcdvd"));
  ratio = (float)checkouts.get("jcdvd") / total;
  text("dvds", left, top+60);
  text(nf(ratio * 100, 0, 1) + "%", left+50, top+60);
  text(checkouts.get("jcdvd") + " c/o", left+125, top+60);
  rect(left+170, top+60, ratio * 250, 15);
  
  fill(colors.get("jcvhs"));
  ratio = (float)checkouts.get("jcvhs") / total;
  text("vhs", left, top+90);
  text(nf(ratio * 100, 0, 1) + "%", left+50, top+90);
  text(checkouts.get("jcvhs") + " c/o", left+125, top+90);
  rect(left+170, top+90, ratio * 250, 15);
  
  // Max, Mean, Median
  noFill();
  strokeWeight(1);
  ellipseMode(CENTER);
  
  top = circle_center_y + 150;
  left = 150;
  
  // Max
  stroke(199, 65, 252);
  ellipse(circle_center_x, circle_center_y, diameter+(2*max), diameter+(2*max));
  ellipse(left+170, top+10, 20, 20);
  
  fill(199, 65, 252);
  text("max", left, top);
  text(int(max) + " checkouts", left+90, top);
  noFill();

  // Mean
  stroke(103, 221, 229);
  ellipse(circle_center_x, circle_center_y, diameter+(2*mean), diameter+(2*mean));
  ellipse(left+170, top+40, 10, 10);
  
  fill(103, 221, 229);
  text("average", left, top+30);
  text(int(mean) + " checkouts", left+90, top+30);
  noFill();
  
  // Median
  stroke(242, 193, 140);
  ellipse(circle_center_x, circle_center_y, diameter+(2*median), diameter+(2*median));
  ellipse(left+170, top+70, 8, 8);
  
  fill(242, 193, 140);
  text("median", left, top+60);
  text(int(median) + " checkouts", left+90, top+60);
  noFill();
  
  // Draw the circle.
  float x1, y1, x2, y2;
  
  for(int i = 0; i < rows; i++) {
    x1 = (diameter/2)*cos(((float)i / rows) * TWO_PI) + circle_center_x;
    y1 = (diameter/2)*sin(((float)i / rows) * TWO_PI) + circle_center_y;
    x2 = ((diameter/2)+(itemCheckouts[i]))*cos(((float)i / rows) * TWO_PI) + circle_center_x;
    y2 = ((diameter/2)+(itemCheckouts[i]))*sin(((float)i / rows) * TWO_PI) + circle_center_y;
    stroke(colors.get(itemType[i]));
    strokeWeight(2);
    line(x1, y1, x2, y2);
  }
  
  // Draw the arcs.
  noStroke();
  String current_type = itemType[0];
  float beginning_rad = 0.0;
  for(int i = 0; i < rows; i++) {
    if(!itemType[i].equals(current_type)) {
      fill(colors.get(itemType[i - 1]));
      arc(circle_center_x, circle_center_y, diameter, diameter, beginning_rad, ((float)(i - 1) / rows) * TWO_PI, PIE);
      beginning_rad = ((float)(i - 1) / rows) * TWO_PI;
      current_type = itemType[i];
    }
  }
  
  fill(colors.get(itemType[rows - 1]));
  arc(circle_center_x, circle_center_y, diameter, diameter, beginning_rad, TWO_PI, PIE);
  
  // Show the count at the mouse position.
  stroke(255);
  strokeWeight(1);
  line(mouseX, mouseY, circle_center_x, circle_center_y);
  noStroke();
  
  float angle = degrees(atan2(0, 1)-atan2(mouseY-circle_center_y, mouseX-circle_center_x));
  if(angle < 0) angle += 360.0;
  
  fill(255);
  text((int)itemCheckouts[itemCheckouts.length - ((int)(angle / 360 * itemCheckouts.length) + 1)], mouseX + 20, mouseY - 10);
}